/**
  ******************************************************************************
  * File Name          : akm.c
  * Description        : AKM common functions
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32f0xx_hal.h"

#include "akm.h"
#include "ak4118.h"
#include <stdio.h>
#include <string.h>

I2C_HandleTypeDef akm_i2c;

HAL_StatusTypeDef write_register(int devaddr, uint8_t regaddr, uint8_t value)  // write value to a data register
{
	uint8_t buf[2];
	buf[0] = regaddr;
	buf[1] = value;
	return HAL_I2C_Master_Transmit(&akm_i2c, devaddr, buf, 2, 10);
}

HAL_StatusTypeDef read_register(int devaddr, uint8_t regaddr, uint8_t *value)  // Read a data register value
{                              
	HAL_StatusTypeDef status = HAL_I2C_Master_Transmit(&akm_i2c, devaddr, &regaddr, 1, 10);
	if (status != HAL_OK) {
		return status;
	}
	return HAL_I2C_Master_Receive(&akm_i2c, devaddr, value, 1, 10);
}

/**
  * @brief  Initializes the I2C source clock and IOs used to drive AKM chips
  * @param  None
  * @retval None
  */
void AKM_LowLevel_Init(void)
{
    GPIO_InitTypeDef  GPIO_InitStruct = {0};

    __HAL_RCC_GPIOB_CLK_ENABLE();

    /*Configure AK4118 PDN pin */
    GPIO_InitStruct.Pin = AK4118_PDN_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Speed = GPIO_SPEED_LOW;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    HAL_GPIO_Init(AK4118_PDN_PORT, &GPIO_InitStruct);

    /*Configure AK4118 INT0 & INT1 GPIO pins */
    GPIO_InitStruct.Pin = AK4118_INT0_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING_FALLING;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    HAL_GPIO_Init(AK4118_INT0_PORT, &GPIO_InitStruct);
    GPIO_InitStruct.Pin = AK4118_INT1_PIN;
    HAL_GPIO_Init(AK4118_INT1_PORT, &GPIO_InitStruct);

    /* SCL and SDA pins */
    GPIO_InitStruct.Pin = AKM_I2C_SCL_PIN;
    GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
    GPIO_InitStruct.Alternate = AKM_I2C_SCL_AF;
    HAL_GPIO_Init(AKM_I2C_SCL_GPIO_PORT, &GPIO_InitStruct);
    GPIO_InitStruct.Pin = AKM_I2C_SDA_PIN;
    GPIO_InitStruct.Alternate = AKM_I2C_SDA_AF;
    HAL_GPIO_Init(AKM_I2C_SDA_GPIO_PORT, &GPIO_InitStruct);
    /* Peripheral clock enable */
    __HAL_RCC_I2C2_CLK_ENABLE();

    HAL_NVIC_SetPriority(EXTI4_15_IRQn, 0, 0);
    HAL_NVIC_EnableIRQ(EXTI4_15_IRQn);
}

/**
  * @brief AKM I2C Initialization Function
  * @param None
  * @retval None
  */
static void AKM_I2C_Init(void)
{
    akm_i2c.Instance = AKM_I2C;
    akm_i2c.Init.Timing = AKM_I2C_TIMING;
    akm_i2c.Init.OwnAddress1 = 0;
    akm_i2c.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
    akm_i2c.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
    akm_i2c.Init.OwnAddress2 = 0;
    akm_i2c.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
    akm_i2c.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
    akm_i2c.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
    if (HAL_I2C_Init(&akm_i2c) != HAL_OK) {
        Error_Handler();
    }
    /* Configure Analogue filter */
    if (HAL_I2CEx_ConfigAnalogFilter(&akm_i2c, I2C_ANALOGFILTER_ENABLE) != HAL_OK) {
        Error_Handler();
    }
    /* Configure Digital filter */
    if (HAL_I2CEx_ConfigDigitalFilter(&akm_i2c, 0) != HAL_OK) {
        Error_Handler();
    }
}

/**
  * @brief  Initializes the AKM I2Cs
  * @param  None
  * @retval None
  */
void AKM_Init(void)
{
	AKM_LowLevel_Init();
	AKM_I2C_Init();
	/* Reset AKM chips */
	HAL_Delay(5);
	HAL_GPIO_WritePin(AK4118_PDN_PORT, AK4118_PDN_PIN, GPIO_PIN_RESET); //PDN low
	HAL_Delay(10);
	HAL_GPIO_WritePin(AK4118_PDN_PORT, AK4118_PDN_PIN, GPIO_PIN_SET); //PDN high
	HAL_Delay(10);
	ak4118_Init();
}
